import subprocess
import sys
from typing import List, Optional
import requests
from packaging.version import parse as parse_version
from pkg_resources import get_distribution
import json
import logging
import pkg_resources
from .install import install_package

def last_version(package: str):
    """Check if last version"""
    url = f'https://pypi.org/pypi/{package}/json'    
    req = requests.get(url)
    version = parse_version('0')
    if req.status_code == requests.codes.ok:
        j = json.loads(req.text.encode(req.encoding))
        releases = j.get('releases', [])
        for release in releases:
            ver = parse_version(release)
            if not ver.is_prerelease:
                version = max(version, ver)
    return version

def check_last(package: str) -> Optional[str]:
    pypi = last_version(package)
    current = parse_version(get_distribution(package).version)
    if pypi > current:
        return pypi


def check_last_masterdac(args: List[str]):
    better_version = check_last("master_dac")

    if better_version:
        logging.info("Updating the package")
        requirement, = pkg_resources.parse_requirements(f"master_dac=={better_version}")
        install_package(requirement)
        logging.info("Updating")
        subprocess.check_call(args)
        sys.exit()

        


from distutils.version import Version
from functools import lru_cache
import json
import logging
import re
import subprocess
import pkg_resources
from typing import Set
import sys
import distutils.spawn
from packaging.version import parse as parse_version
from pkg_resources import Requirement
import easypip

try:
    from importlib.resources import files as resources_files
except:
    from importlib_resources import files as resources_files

@lru_cache()
def cuda_version() -> Version:
    try:
        re_cuda = re.compile(br".*CUDA version: ([\d\.]+)", re.IGNORECASE)
        out = subprocess.check_output("nvidia-smi")
        for line in out.splitlines():
            m = re_cuda.match(line)
            if m:
                return parse_version(m.group(1).decode("utf-8"))
    except:
        pass
    logging.info("No CUDA detected")


def install_package(requirement: Requirement):
    if easypip.has_requirement(requirement):
        logging.info("Package %s is already installed", requirement)
        return

    easypip.install(requirement)

def install(name: str, processed: Set[str]):
    if name in processed:
        return

    path = resources_files("master_dac") / "requirements" / f"{name}.txt"

    for value in pkg_resources.parse_requirements(path.read_text()):
        if not value.marker or value.marker.evaluate():
            install_package(value)
        else:
            logging.info("Skipping %s", value)

    processed.add(name)



def rld(processed: Set[str]):
    # Check that swig is installed
    if sys.platform == "win32":
        has_swig = distutils.spawn.find_executable("swig.exe")
    else:
        has_swig = distutils.spawn.find_executable("swig")

    if not has_swig:
        logging.error("swig n'est pas installé: sous linux utilisez le gestionnaire de paquets:\n - sous windows/conda : conda install swig\n - sous ubuntu: sudo apt install swig")
        sys.exit(1)


    install("deep", processed)
    install("rld", processed)

def amal(processed: Set[str]):
    install("deep", processed)
    install("amal", processed)

def rital(processed: Set[str]):
    install("deep", processed)
    install("rital", processed)



import importlib
from typing import List
import click
import sys
import logging
logging.basicConfig(level=logging.INFO)
from .configuration import Configuration
from .utils import check_last_masterdac

original_args = [arg for arg in sys.argv]

@click.group()
def main():
    pass


@main.group()
def courses():
    """Permet de gérer la liste des cours suivis"""
    pass

@click.argument("courses", type=click.Choice(["amal", "rld", "rital"]), nargs=-1)
@courses.command("add")
def courses_add(courses: List[str]):
    """Ajout de cours"""
    configuration = Configuration()
    configuration.courses.update(courses)
    configuration.save()

@click.argument("courses", type=click.Choice(["amal", "rld", "rital"]), nargs=-1)
@courses.command("rm")
def courses_rm(courses: List[str]):
    """Enlever un cours"""
    configuration = Configuration()
    for course in courses:
        configuration.courses.remove(course)
    configuration.save()


@courses.command("list")
def courses_list():
    """Liste des cours"""
    for course in Configuration().courses:
        print(course)

@click.option("--no-self-update", is_flag=True)
@main.command()
def update(no_self_update: bool):
    """Mettre à jour l'ensemble des modules pour les cours suivis"""
    if not no_self_update:
        check_last_masterdac(original_args)

    # Check that package are installed

    configuration = Configuration()
    install = importlib.import_module("master_dac.install")

    processed = set()
    for course in configuration.courses:
        getattr(install, course)(processed)

    print("Don't forget to download the datasets with `master-dac download-datasets`")

@main.command()
def download_datasets():
    """Mettre à jour l'ensemble des jeux de données pour les cours suivis"""
    configuration = Configuration()
    install = importlib.import_module("master_dac.datasets")

    for course in configuration.courses:
        getattr(install, course, lambda *args: None)()



from setuptools import setup
import os
import sys

def is_venv():
    return (
        hasattr(sys, 'real_prefix') or
        (hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix) or
        (os.environ.get("CONDA_PREFIX", None) is not None)
    )

if not is_venv():
    print("WARNING: you should be installing this package within a virtual environment", file=sys.stderr)
    print("(if running from google colab, ignore this message)", file=sys.stderr)

# Let's go
setup()
